{-# LANGUAGE MagicHash #-}
{-# LANGUAGE ScopedTypeVariables #-}
{-# OPTIONS_GHC -fno-warn-overlapping-patterns #-}

module Rewriting.Curry_Term
  ( C_VarIdx
  , C_TermEq
  , C_TermEqs
  , C_Term (..)
  , d_C_showVarIdx
  , d_C_showTerm
  , nd_C_showTerm
  , d_C_showTermEq
  , nd_C_showTermEq
  , d_C_showTermEqs
  , nd_C_showTermEqs
  , d_C_tConst
  , d_C_tOp
  , d_C_tRoot
  , d_C_tCons
  , d_C_tConsAll
  , d_C_tVars
  , d_C_tVarsAll
  , d_C_isConsTerm
  , d_C_isVarTerm
  , d_C_isGround
  , d_C_isLinear
  , d_C_isNormal
  , d_C_maxVarInTerm
  , d_C_minVarInTerm
  , d_C_normalizeTerm
  , d_C_renameTermVars
  , d_C_mapTerm
  , nd_C_mapTerm
  , d_C_eqConsPattern )
  where

import Basics
import qualified Curry_FiniteMap
import qualified Curry_List
import qualified Curry_Maybe
import qualified Curry_Prelude


type C_VarIdx = Curry_Prelude.C_Int

type C_TermEq t0 = Curry_Prelude.OP_Tuple2 (C_Term t0) (C_Term t0)

type C_TermEqs t0 = Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 (C_Term t0)
  (C_Term t0))

data C_Term t0
  = C_TermVar Curry_Prelude.C_Int
  | C_TermCons t0 (Curry_Prelude.OP_List (C_Term t0))
  | Choice_C_Term Cover ID (C_Term t0) (C_Term t0)
  | Choices_C_Term Cover ID [C_Term t0]
  | Fail_C_Term Cover FailInfo
  | Guard_C_Term Cover Constraints (C_Term t0)

instance (Show t0) => Show (C_Term t0) where
  showsPrec d (Choice_C_Term cd i x y) = showsChoice d cd i x y
  showsPrec d (Choices_C_Term cd i xs) = showsChoices d cd i xs
  showsPrec d (Guard_C_Term cd c e) = showsGuard d cd c e
  showsPrec _ (Fail_C_Term cd info) = showChar '!'
  showsPrec _ (C_TermVar x1) = showString "(TermVar" . (showChar ' ' . (shows
    x1 . showChar ')'))
  showsPrec _ (C_TermCons x1 x2) = showString "(TermCons" . (showChar ' ' .
    (shows x1 . (showChar ' ' . (shows x2 . showChar ')'))))

instance (Read t0) => Read (C_Term t0) where
  readsPrec d s = readParen ((>) d 10) (\r -> [(C_TermVar x1, r1) | (_, r0) <-
    readQualified "Rewriting.Term" "TermVar" r, (x1, r1) <- readsPrec 11 r0])
    s ++ readParen ((>) d 10) (\r -> [(C_TermCons x1 x2, r2) | (_, r0) <-
    readQualified "Rewriting.Term" "TermCons" r, (x1, r1) <- readsPrec 11 r0,
    (x2, r2) <- readsPrec 11 r1]) s

instance NonDet (C_Term t0) where
  choiceCons = Choice_C_Term
  choicesCons = Choices_C_Term
  failCons = Fail_C_Term
  guardCons = Guard_C_Term
  try (Choice_C_Term cd i x y) = tryChoice cd i x y
  try (Choices_C_Term cd i xs) = tryChoices cd i xs
  try (Fail_C_Term cd info) = Fail cd info
  try (Guard_C_Term cd c e) = Guard cd c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_Term cd i x y) = f cd i x y
  match _ f _ _ _ _ (Choices_C_Term cd i@(NarrowedID _ _) xs) = f cd i xs
  match _ _ f _ _ _ (Choices_C_Term cd i@(FreeID _ _) xs) = f cd i xs
  match _ _ _ _ _ _ (Choices_C_Term cd i _) = error
    ("Rewriting.Term.Term.match: Choices with ChoiceID " ++ show i)
  match _ _ _ f _ _ (Fail_C_Term cd info) = f cd info
  match _ _ _ _ f _ (Guard_C_Term cd c e) = f cd c e
  match _ _ _ _ _ f x = f x

instance (Generable t0) => Generable (C_Term t0) where
  generate s c = Choices_C_Term c (freeID [1, 2] s) [C_TermVar (generate
    (leftSupply s) c), C_TermCons (generate (leftSupply s) c) (generate
    (rightSupply s) c)]

instance (NormalForm t0) => NormalForm (C_Term t0) where
  ($!!) cont (C_TermVar x1) d cs = ($!!) (\y1 d cs -> cont (C_TermVar y1) d cs)
    x1 d cs
  ($!!) cont (C_TermCons x1 x2) d cs = ($!!) (\y1 d cs -> ($!!) (\y2 d cs ->
    cont (C_TermCons y1 y2) d cs) x2 d cs) x1 d cs
  ($!!) cont (Choice_C_Term cd i x y) d cs = nfChoice cont cd i x y cd cs
  ($!!) cont (Choices_C_Term cd i xs) d cs = nfChoices cont cd i xs d cs
  ($!!) cont (Guard_C_Term cd c e) d cs = guardCons cd c (($!!) cont e d (addCs
    c cs))
  ($!!) _ (Fail_C_Term cd info) _ _ = failCons cd info
  ($##) cont (C_TermVar x1) d cs = ($##) (\y1 d cs -> cont (C_TermVar y1) d cs)
    x1 d cs
  ($##) cont (C_TermCons x1 x2) d cs = ($##) (\y1 d cs -> ($##) (\y2 d cs ->
    cont (C_TermCons y1 y2) d cs) x2 d cs) x1 d cs
  ($##) cont (Choice_C_Term cd i x y) d cs = gnfChoice cont cd i x y cd cs
  ($##) cont (Choices_C_Term cd i xs) d cs = gnfChoices cont cd i xs d cs
  ($##) cont (Guard_C_Term cd c e) d cs = guardCons cd c (($##) cont e d (addCs
    c cs))
  ($##) _ (Fail_C_Term cd info) _ _ = failCons cd info
  showCons (C_TermVar _) = "Rewriting.Term.TermVar _"
  showCons (C_TermCons _ _) = "Rewriting.Term.TermCons _ _"
  showCons x = error ("Rewriting.Term.Term.showCons: no constructor: " ++ show
    x)
  searchNF search cont (C_TermVar x1) = search (\y1 -> cont (C_TermVar y1)) x1
  searchNF search cont (C_TermCons x1 x2) = search (\y1 -> search (\y2 -> cont
    (C_TermCons y1 y2)) x2) x1
  searchNF _ _ x = error ("Rewriting.Term.Term.searchNF: no constructor: " ++
    show x)

instance (Unifiable t0) => Unifiable (C_Term t0) where
  (=.=) (C_TermVar x1) (C_TermVar y1) d cs = (=:=) x1 y1 d cs
  (=.=) (C_TermCons x1 x2) (C_TermCons y1 y2) d cs = (&) ((=:=) x1 y1 d cs)
    ((=:=) x2 y2 d cs) d cs
  (=.=) a b cd _ = Fail_C_Bool cd (unificationFail (showCons a) (showCons b))
  (=.<=) (C_TermVar x1) (C_TermVar y1) d cs = (=:<=) x1 y1 d cs
  (=.<=) (C_TermCons x1 x2) (C_TermCons y1 y2) d cs = (&) ((=:<=) x1 y1 d cs)
    ((=:<=) x2 y2 d cs) d cs
  (=.<=) a b cd _ = Fail_C_Bool cd (unificationFail (showCons a) (showCons b))
  bind cd i (C_TermVar x3) = (i :=: ChooseN 0 1) : concat [bind cd (leftID i)
    x3]
  bind cd i (C_TermCons x3 x4) = (i :=: ChooseN 1 2) : concat [bind cd (leftID
    i) x3, bind cd (rightID i) x4]
  bind d i (Choice_C_Term cd j x y) = [ConstraintChoice cd j (bind d i x) (bind
    d i y)]
  bind d i (Choices_C_Term cd j@(FreeID _ _) xs) = bindOrNarrow d i cd j xs
  bind d i (Choices_C_Term cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd j
    (map (bind d i) xs)]
  bind _ _ (Choices_C_Term cd i _) = error
    ("Rewriting.Term.Term.bind: Choices with ChoiceID: " ++ show i)
  bind _ _ (Fail_C_Term cd info) = [Unsolvable info]
  bind d i (Guard_C_Term cd c e) = getConstrList c ++ bind d i e
  lazyBind cd i (C_TermVar x3) = (i :=: ChooseN 0 1) : [leftID i :=: LazyBind
    (lazyBind cd (leftID i) x3)]
  lazyBind cd i (C_TermCons x3 x4) = (i :=: ChooseN 1 2) : [leftID i :=:
    LazyBind (lazyBind cd (leftID i) x3), rightID i :=: LazyBind (lazyBind cd
    (rightID i) x4)]
  lazyBind d i (Choice_C_Term cd j x y) = [ConstraintChoice cd j (lazyBind d i
    x) (lazyBind d i y)]
  lazyBind d i (Choices_C_Term cd j@(FreeID _ _) xs) = lazyBindOrNarrow d i cd j
    xs
  lazyBind d i (Choices_C_Term cd j@(NarrowedID _ _) xs) = [ConstraintChoices cd
    j (map (lazyBind d i) xs)]
  lazyBind _ _ (Choices_C_Term cd i _) = error
    ("Rewriting.Term.Term.lazyBind: Choices with ChoiceID: " ++ show i)
  lazyBind _ _ (Fail_C_Term cd info) = [Unsolvable info]
  lazyBind d i (Guard_C_Term cd c e) = getConstrList c ++ [i :=: LazyBind
    (lazyBind d i e)]

instance (Curry t0) => Curry (C_Term t0) where
  (=?=) (Choice_C_Term cd i x y) z d cs = narrow cd i ((=?=) x z d cs) ((=?=) y
    z d cs)
  (=?=) (Choices_C_Term cd i xs) y d cs = narrows cs cd i (\x -> (=?=) x y d cs)
    xs
  (=?=) (Guard_C_Term cd c e) y d cs = guardCons cd c ((=?=) e y d (addCs c cs))
  (=?=) (Fail_C_Term cd info) _ _ _ = failCons cd info
  (=?=) z (Choice_C_Term cd i x y) d cs = narrow cd i ((=?=) z x d cs) ((=?=) z
    y d cs)
  (=?=) y (Choices_C_Term cd i xs) d cs = narrows cs cd i (\x -> (=?=) y x d cs)
    xs
  (=?=) y (Guard_C_Term cd c e) d cs = guardCons cd c ((=?=) y e d (addCs c cs))
  (=?=) _ (Fail_C_Term cd info) _ _ = failCons cd info
  (=?=) (C_TermVar x1) (C_TermVar y1) d cs = (=?=) x1 y1 d cs
  (=?=) (C_TermCons x1 x2) (C_TermCons y1 y2) d cs = Curry_Prelude.d_OP_amp_amp
    ((=?=) x1 y1 d cs) ((=?=) x2 y2 d cs) d cs
  (=?=) _ _ d _ = C_False
  (<?=) (Choice_C_Term cd i x y) z d cs = narrow cd i ((<?=) x z d cs) ((<?=) y
    z d cs)
  (<?=) (Choices_C_Term cd i xs) y d cs = narrows cs cd i (\x -> (<?=) x y d cs)
    xs
  (<?=) (Guard_C_Term cd c e) y d cs = guardCons cd c ((<?=) e y d (addCs c cs))
  (<?=) (Fail_C_Term cd info) _ _ _ = failCons cd info
  (<?=) z (Choice_C_Term cd i x y) d cs = narrow cd i ((<?=) z x d cs) ((<?=) z
    y d cs)
  (<?=) y (Choices_C_Term cd i xs) d cs = narrows cs cd i (\x -> (<?=) y x d cs)
    xs
  (<?=) y (Guard_C_Term cd c e) d cs = guardCons cd c ((<?=) y e d (addCs c cs))
  (<?=) _ (Fail_C_Term cd info) _ _ = failCons cd info
  (<?=) (C_TermVar x1) (C_TermVar y1) d cs = (<?=) x1 y1 d cs
  (<?=) (C_TermVar _) (C_TermCons _ _) _ _ = C_True
  (<?=) (C_TermCons x1 x2) (C_TermCons y1 y2) d cs = Curry_Prelude.d_OP_bar_bar
    (Curry_Prelude.d_OP_lt x1 y1 d cs) (Curry_Prelude.d_OP_amp_amp ((=?=) x1 y1
    d cs) ((<?=) x2 y2 d cs) d cs) d cs
  (<?=) _ _ d _ = C_False

d_C_showVarIdx :: Curry_Prelude.C_Int -> Cover -> ConstStore
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showVarIdx x1 cd cs = let x2 = Curry_Prelude.d_C_divMod x1
                                (Curry_Prelude.C_Int 26#) cd cs
                              x3 = d_OP_showVarIdx_dot_uscore_hash_selFP2_hash_q
                                x2 cd cs
                              x4 = d_OP_showVarIdx_dot_uscore_hash_selFP3_hash_r
                                x2 cd cs
                              x5 = Curry_Prelude.d_OP_bang_bang
                                (Curry_Prelude.toCurryString
                                "abcdefghijklmnopqrstuvwxyz") x4 cd cs
  in d_C__case_12 x1 x3 x5 (Curry_Prelude.d_OP_gt_eq x1 (Curry_Prelude.C_Int 0#)
  cd cs) cd cs

d_OP_showVarIdx_dot_uscore_hash_selFP2_hash_q :: Curry_Prelude.OP_Tuple2
  Curry_Prelude.C_Int Curry_Prelude.C_Int -> Cover -> ConstStore
  -> Curry_Prelude.C_Int
d_OP_showVarIdx_dot_uscore_hash_selFP2_hash_q x1 cd cs = case x1 of
  Curry_Prelude.OP_Tuple2 x2 x3 -> x2
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i
    (d_OP_showVarIdx_dot_uscore_hash_selFP2_hash_q l cd cs)
    (d_OP_showVarIdx_dot_uscore_hash_selFP2_hash_q r cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z ->
    d_OP_showVarIdx_dot_uscore_hash_selFP2_hash_q z cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c
    (d_OP_showVarIdx_dot_uscore_hash_selFP2_hash_q e cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "Rewriting.Term.showVarIdx._#selFP2#q" [show x1] info)
  _ -> failCons cd (consFail "Rewriting.Term.showVarIdx._#selFP2#q" (showCons
    x1))

d_OP_showVarIdx_dot_uscore_hash_selFP3_hash_r :: Curry_Prelude.OP_Tuple2
  Curry_Prelude.C_Int Curry_Prelude.C_Int -> Cover -> ConstStore
  -> Curry_Prelude.C_Int
d_OP_showVarIdx_dot_uscore_hash_selFP3_hash_r x1 cd cs = case x1 of
  Curry_Prelude.OP_Tuple2 x2 x3 -> x3
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i
    (d_OP_showVarIdx_dot_uscore_hash_selFP3_hash_r l cd cs)
    (d_OP_showVarIdx_dot_uscore_hash_selFP3_hash_r r cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z ->
    d_OP_showVarIdx_dot_uscore_hash_selFP3_hash_r z cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c
    (d_OP_showVarIdx_dot_uscore_hash_selFP3_hash_r e cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "Rewriting.Term.showVarIdx._#selFP3#r" [show x1] info)
  _ -> failCons cd (consFail "Rewriting.Term.showVarIdx._#selFP3#r" (showCons
    x1))

d_C_showTerm :: forall t0 . (Curry_Prelude.Curry t0) => (t0 -> Cover
  -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover
  -> ConstStore -> C_Term t0 -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
d_C_showTerm x1 cd cs = d_OP_showTerm_dot_showTerm_tick_dot_9 x1
  Curry_Prelude.C_False

nd_C_showTerm :: forall t0 . (Curry_Prelude.Curry t0) => Func t0
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> IDSupply -> Cover
  -> ConstStore -> Func (C_Term t0) (Curry_Prelude.OP_List Curry_Prelude.C_Char)
nd_C_showTerm x1 s cd cs = wrapNX id (nd_OP_showTerm_dot_showTerm_tick_dot_9 x1
  Curry_Prelude.C_False)

d_OP_showTerm_dot_showTerm_tick_dot_9 :: forall t0 . (Curry_Prelude.Curry
  t0) => (t0 -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char) -> Curry_Prelude.C_Bool -> C_Term t0 -> Cover
  -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_OP_showTerm_dot_showTerm_tick_dot_9 x1 x2 x3 cd cs = case x3 of
  C_TermVar x4 -> d_C_showVarIdx x4 cd cs
  C_TermCons x5 x6 -> d_C__case_9 x1 x5 x2 x6 cd cs
  Choice_C_Term d i l r -> narrow d i (d_OP_showTerm_dot_showTerm_tick_dot_9 x1
    x2 l cd cs) (d_OP_showTerm_dot_showTerm_tick_dot_9 x1 x2 r cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z ->
    d_OP_showTerm_dot_showTerm_tick_dot_9 x1 x2 z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_OP_showTerm_dot_showTerm_tick_dot_9 x1
    x2 e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail
    "Rewriting.Term.showTerm.showTerm'.9" [show x1, show x2, show x3] info)
  _ -> failCons cd (consFail "Rewriting.Term.showTerm.showTerm'.9" (showCons
    x3))

nd_OP_showTerm_dot_showTerm_tick_dot_9 :: forall t0 . (Curry_Prelude.Curry
  t0) => Func t0 (Curry_Prelude.OP_List Curry_Prelude.C_Char)
  -> Curry_Prelude.C_Bool -> C_Term t0 -> IDSupply -> Cover -> ConstStore
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char
nd_OP_showTerm_dot_showTerm_tick_dot_9 x1 x2 x3 s cd cs = case x3 of
  C_TermVar x4 -> d_C_showVarIdx x4 cd cs
  C_TermCons x5 x6 -> let s0 = s in s0 `seq` nd_C__case_9 x1 x5 x2 x6 s0 cd cs
  Choice_C_Term d i l r -> narrow d i (nd_OP_showTerm_dot_showTerm_tick_dot_9 x1
    x2 l s cd cs) (nd_OP_showTerm_dot_showTerm_tick_dot_9 x1 x2 r s cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z ->
    nd_OP_showTerm_dot_showTerm_tick_dot_9 x1 x2 z s cd cs) xs
  Guard_C_Term d c e -> guardCons d c (nd_OP_showTerm_dot_showTerm_tick_dot_9 x1
    x2 e s cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail
    "Rewriting.Term.showTerm.showTerm'.9" [show x1, show x2, show x3] info)
  _ -> failCons cd (consFail "Rewriting.Term.showTerm.showTerm'.9" (showCons
    x3))

d_C_showTermEq :: forall t0 . (Curry_Prelude.Curry t0) => (t0 -> Cover
  -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char)
  -> Curry_Prelude.OP_Tuple2 (C_Term t0) (C_Term t0) -> Cover -> ConstStore
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C_showTermEq x1 x2 cd cs = case x2 of
  Curry_Prelude.OP_Tuple2 x3 x4 -> Curry_Prelude.d_OP_plus_plus
    (Curry_Prelude.d_C_apply (d_C_showTerm x1 cd cs) x3 cd cs)
    (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " = ")
    (Curry_Prelude.d_C_apply (d_C_showTerm x1 cd cs) x4 cd cs) cd cs) cd cs
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i (d_C_showTermEq x1 l cd
    cs) (d_C_showTermEq x1 r cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z -> d_C_showTermEq
    x1 z cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c (d_C_showTermEq x1 e
    cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "Rewriting.Term.showTermEq" [show x1, show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term.showTermEq" (showCons x2))

nd_C_showTermEq :: forall t0 . (Curry_Prelude.Curry t0) => Func t0
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Curry_Prelude.OP_Tuple2
  (C_Term t0) (C_Term t0) -> IDSupply -> Cover -> ConstStore
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char
nd_C_showTermEq x1 x2 s cd cs = case x2 of
  Curry_Prelude.OP_Tuple2 x3 x4 -> let s6 = s
    in s6 `seq` (let s2 = leftSupply s6
                     s5 = rightSupply s6
    in s2 `seq` (s5 `seq` Curry_Prelude.d_OP_plus_plus (let s1 = leftSupply s2
                                                            s0 = rightSupply s2
    in s1 `seq` (s0 `seq` Curry_Prelude.nd_C_apply (nd_C_showTerm x1 s0 cd cs)
    x3 s1 cd cs)) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString
    " = ") (let s4 = leftSupply s5
                s3 = rightSupply s5
    in s4 `seq` (s3 `seq` Curry_Prelude.nd_C_apply (nd_C_showTerm x1 s3 cd cs)
    x4 s4 cd cs)) cd cs) cd cs))
  Curry_Prelude.Choice_OP_Tuple2 d i l r -> narrow d i (nd_C_showTermEq x1 l s
    cd cs) (nd_C_showTermEq x1 r s cd cs)
  Curry_Prelude.Choices_OP_Tuple2 d i xs -> narrows cs d i (\z ->
    nd_C_showTermEq x1 z s cd cs) xs
  Curry_Prelude.Guard_OP_Tuple2 d c e -> guardCons d c (nd_C_showTermEq x1 e s
    cd $! addCs c cs)
  Curry_Prelude.Fail_OP_Tuple2 d info -> failCons d (traceFail
    "Rewriting.Term.showTermEq" [show x1, show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term.showTermEq" (showCons x2))

d_C_showTermEqs :: forall t0 . (Curry_Prelude.Curry t0) => (t0 -> Cover
  -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char) -> Cover
  -> ConstStore -> Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 (C_Term t0)
  (C_Term t0)) -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
d_C_showTermEqs x1 cd cs = Curry_Prelude.d_OP_dot Curry_Prelude.d_C_unlines
  (Curry_Prelude.d_C_map (d_C_showTermEq x1)) cd cs

nd_C_showTermEqs :: forall t0 . (Curry_Prelude.Curry t0) => Func t0
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> IDSupply -> Cover
  -> ConstStore -> Func (Curry_Prelude.OP_List (Curry_Prelude.OP_Tuple2 (C_Term
  t0) (C_Term t0))) (Curry_Prelude.OP_List Curry_Prelude.C_Char)
nd_C_showTermEqs x1 s cd cs = let s0 = s
  in s0 `seq` Curry_Prelude.nd_OP_dot (wrapDX id Curry_Prelude.d_C_unlines)
  (wrapNX id (Curry_Prelude.nd_C_map (wrapNX id (nd_C_showTermEq x1)))) s0 cd cs

d_C_tConst :: forall t0 . (Curry_Prelude.Curry t0) => t0 -> Cover -> ConstStore
  -> C_Term t0
d_C_tConst x1 cd cs = C_TermCons x1 Curry_Prelude.OP_List

d_C_tOp :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> t0 -> C_Term t0
  -> Cover -> ConstStore -> C_Term t0
d_C_tOp x1 x2 x3 cd cs = C_TermCons x2 (Curry_Prelude.OP_Cons x1
  (Curry_Prelude.OP_Cons x3 Curry_Prelude.OP_List))

d_C_tRoot :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> Cover
  -> ConstStore -> Curry_Prelude.C_Either Curry_Prelude.C_Int t0
d_C_tRoot x1 cd cs = case x1 of
  C_TermVar x2 -> Curry_Prelude.C_Left x2
  C_TermCons x3 x4 -> Curry_Prelude.C_Right x3
  Choice_C_Term d i l r -> narrow d i (d_C_tRoot l cd cs) (d_C_tRoot r cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C_tRoot z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_C_tRoot e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term.tRoot" [show x1]
    info)
  _ -> failCons cd (consFail "Rewriting.Term.tRoot" (showCons x1))

d_C_tCons :: forall t0 . (Curry_Prelude.Curry t0) => Cover -> ConstStore
  -> C_Term t0 -> Cover -> ConstStore -> Curry_Prelude.OP_List t0
d_C_tCons cd cs = Curry_Prelude.d_OP_dot Curry_List.d_C_nub d_C_tConsAll cd cs

d_C_tConsAll :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> Cover
  -> ConstStore -> Curry_Prelude.OP_List t0
d_C_tConsAll x1 cd cs = case x1 of
  C_TermVar x2 -> Curry_Prelude.OP_List
  C_TermCons x3 x4 -> Curry_Prelude.OP_Cons x3 (Curry_Prelude.d_C_apply
    (Curry_Prelude.d_C_concatMap d_C_tConsAll cd cs) x4 cd cs)
  Choice_C_Term d i l r -> narrow d i (d_C_tConsAll l cd cs) (d_C_tConsAll r cd
    cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C_tConsAll z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_C_tConsAll e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term.tConsAll" [show
    x1] info)
  _ -> failCons cd (consFail "Rewriting.Term.tConsAll" (showCons x1))

d_C_tVars :: forall t0 . (Curry_Prelude.Curry t0) => Cover -> ConstStore
  -> C_Term t0 -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Int
d_C_tVars cd cs = Curry_Prelude.d_OP_dot Curry_List.d_C_nub d_C_tVarsAll cd cs

d_C_tVarsAll :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> Cover
  -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Int
d_C_tVarsAll x1 cd cs = case x1 of
  C_TermVar x2 -> Curry_Prelude.OP_Cons x2 Curry_Prelude.OP_List
  C_TermCons x3 x4 -> Curry_Prelude.d_C_apply (Curry_Prelude.d_C_concatMap
    d_C_tVarsAll cd cs) x4 cd cs
  Choice_C_Term d i l r -> narrow d i (d_C_tVarsAll l cd cs) (d_C_tVarsAll r cd
    cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C_tVarsAll z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_C_tVarsAll e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term.tVarsAll" [show
    x1] info)
  _ -> failCons cd (consFail "Rewriting.Term.tVarsAll" (showCons x1))

d_C_isConsTerm :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> Cover
  -> ConstStore -> Curry_Prelude.C_Bool
d_C_isConsTerm x1 cd cs = case x1 of
  C_TermVar x2 -> Curry_Prelude.C_False
  C_TermCons x3 x4 -> Curry_Prelude.C_True
  Choice_C_Term d i l r -> narrow d i (d_C_isConsTerm l cd cs) (d_C_isConsTerm r
    cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C_isConsTerm z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_C_isConsTerm e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term.isConsTerm" [show
    x1] info)
  _ -> failCons cd (consFail "Rewriting.Term.isConsTerm" (showCons x1))

d_C_isVarTerm :: forall t0 . (Curry_Prelude.Curry t0) => Cover -> ConstStore
  -> C_Term t0 -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isVarTerm cd cs = Curry_Prelude.d_OP_dot Curry_Prelude.d_C_not
  d_C_isConsTerm cd cs

d_C_isGround :: forall t0 . (Curry_Prelude.Curry t0) => Cover -> ConstStore
  -> C_Term t0 -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isGround cd cs = Curry_Prelude.d_OP_dot Curry_Prelude.d_C_null d_C_tVarsAll
  cd cs

d_C_isLinear :: forall t0 . (Curry_Prelude.Curry t0) => Cover -> ConstStore
  -> C_Term t0 -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_isLinear cd cs = Curry_Prelude.d_OP_dot d_C_unique d_C_tVarsAll cd cs

d_C_isNormal :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> Cover
  -> ConstStore -> Curry_Prelude.C_Bool
d_C_isNormal x1 cd cs = case x1 of
  C_TermVar x2 -> Curry_Prelude.C_True
  C_TermCons x3 x4 -> d_C__case_6 x3 x4 cd cs
  Choice_C_Term d i l r -> narrow d i (d_C_isNormal l cd cs) (d_C_isNormal r cd
    cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C_isNormal z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_C_isNormal e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term.isNormal" [show
    x1] info)
  _ -> failCons cd (consFail "Rewriting.Term.isNormal" (showCons x1))

d_C_maxVarInTerm :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> Cover
  -> ConstStore -> Curry_Prelude.C_Maybe Curry_Prelude.C_Int
d_C_maxVarInTerm x1 cd cs = let x2 = Curry_Prelude.d_C_apply (d_C_tVars cd cs)
                                  x1 cd cs
  in d_C__case_4 x2 cd cs

d_C_minVarInTerm :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> Cover
  -> ConstStore -> Curry_Prelude.C_Maybe Curry_Prelude.C_Int
d_C_minVarInTerm x1 cd cs = let x2 = Curry_Prelude.d_C_apply (d_C_tVars cd cs)
                                  x1 cd cs
  in d_C__case_3 x2 cd cs

d_C_normalizeTerm :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> Cover
  -> ConstStore -> C_Term t0
d_C_normalizeTerm x1 cd cs = let x2 = Curry_Prelude.d_C_apply
                                   (Curry_FiniteMap.d_C_listToFM (acceptCs id
                                   Curry_Prelude.d_OP_lt) cd cs)
                                   (Curry_Prelude.d_C_zip
                                   (Curry_Prelude.d_C_apply (d_C_tVars cd cs) x1
                                   cd cs) (Curry_Prelude.d_C_map (acceptCs id
                                   C_TermVar) (Curry_Prelude.d_C_enumFrom
                                   (Curry_Prelude.C_Int 0#) cd cs) cd cs) cd cs)
                                   cd cs
  in d_OP_normalizeTerm_dot_normalize_dot_94 x2 x1 cd cs

d_OP_normalizeTerm_dot_normalize_dot_94 :: forall t0 . (Curry_Prelude.Curry
  t0) => Curry_FiniteMap.C_FM Curry_Prelude.C_Int (C_Term t0) -> C_Term t0
  -> Cover -> ConstStore -> C_Term t0
d_OP_normalizeTerm_dot_normalize_dot_94 x1 x2 cd cs = case x2 of
  C_TermVar x3 -> Curry_Maybe.d_C_fromMaybe x2 (Curry_FiniteMap.d_C_lookupFM x1
    x3 cd cs) cd cs
  C_TermCons x4 x5 -> C_TermCons x4 (Curry_Prelude.d_C_map
    (d_OP_normalizeTerm_dot_normalize_dot_94 x1) x5 cd cs)
  Choice_C_Term d i l r -> narrow d i (d_OP_normalizeTerm_dot_normalize_dot_94
    x1 l cd cs) (d_OP_normalizeTerm_dot_normalize_dot_94 x1 r cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z ->
    d_OP_normalizeTerm_dot_normalize_dot_94 x1 z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_OP_normalizeTerm_dot_normalize_dot_94
    x1 e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail
    "Rewriting.Term.normalizeTerm.normalize.94" [show x1, show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term.normalizeTerm.normalize.94"
    (showCons x2))

nd_OP_normalizeTerm_dot_normalize_dot_94 :: forall t0 . (Curry_Prelude.Curry
  t0) => Curry_FiniteMap.C_FM Curry_Prelude.C_Int (C_Term t0) -> C_Term t0
  -> IDSupply -> Cover -> ConstStore -> C_Term t0
nd_OP_normalizeTerm_dot_normalize_dot_94 x1 x2 s cd cs = case x2 of
  C_TermVar x3 -> let s0 = s
    in s0 `seq` Curry_Maybe.d_C_fromMaybe x2 (Curry_FiniteMap.nd_C_lookupFM x1
    x3 s0 cd cs) cd cs
  C_TermCons x4 x5 -> let s0 = s
    in s0 `seq` C_TermCons x4 (Curry_Prelude.nd_C_map (wrapNX id
    (nd_OP_normalizeTerm_dot_normalize_dot_94 x1)) x5 s0 cd cs)
  Choice_C_Term d i l r -> narrow d i (nd_OP_normalizeTerm_dot_normalize_dot_94
    x1 l s cd cs) (nd_OP_normalizeTerm_dot_normalize_dot_94 x1 r s cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z ->
    nd_OP_normalizeTerm_dot_normalize_dot_94 x1 z s cd cs) xs
  Guard_C_Term d c e -> guardCons d c (nd_OP_normalizeTerm_dot_normalize_dot_94
    x1 e s cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail
    "Rewriting.Term.normalizeTerm.normalize.94" [show x1, show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term.normalizeTerm.normalize.94"
    (showCons x2))

d_C_renameTermVars :: forall t0 . (Curry_Prelude.Curry
  t0) => Curry_Prelude.C_Int -> C_Term t0 -> Cover -> ConstStore -> C_Term t0
d_C_renameTermVars x1 x2 cd cs = case x2 of
  C_TermVar x3 -> C_TermVar (Curry_Prelude.d_OP_plus x3 x1 cd cs)
  C_TermCons x4 x5 -> C_TermCons x4 (Curry_Prelude.d_C_map (d_C_renameTermVars
    x1) x5 cd cs)
  Choice_C_Term d i l r -> narrow d i (d_C_renameTermVars x1 l cd cs)
    (d_C_renameTermVars x1 r cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C_renameTermVars x1 z cd cs)
    xs
  Guard_C_Term d c e -> guardCons d c (d_C_renameTermVars x1 e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term.renameTermVars"
    [show x1, show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term.renameTermVars" (showCons x2))

d_C_mapTerm :: forall t0 t1 . (Curry_Prelude.Curry t0, Curry_Prelude.Curry
  t1) => (t0 -> Cover -> ConstStore -> t1) -> C_Term t0 -> Cover -> ConstStore
  -> C_Term t1
d_C_mapTerm x1 x2 cd cs = case x2 of
  C_TermVar x3 -> C_TermVar x3
  C_TermCons x4 x5 -> C_TermCons (Curry_Prelude.d_C_apply x1 x4 cd cs)
    (Curry_Prelude.d_C_map (d_C_mapTerm x1) x5 cd cs)
  Choice_C_Term d i l r -> narrow d i (d_C_mapTerm x1 l cd cs) (d_C_mapTerm x1 r
    cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C_mapTerm x1 z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_C_mapTerm x1 e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term.mapTerm" [show x1
    , show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term.mapTerm" (showCons x2))

nd_C_mapTerm :: forall t0 t1 . (Curry_Prelude.Curry t0, Curry_Prelude.Curry
  t1) => Func t0 t1 -> C_Term t0 -> IDSupply -> Cover -> ConstStore -> C_Term t1
nd_C_mapTerm x1 x2 s cd cs = case x2 of
  C_TermVar x3 -> C_TermVar x3
  C_TermCons x4 x5 -> let s2 = s
    in s2 `seq` (let s0 = leftSupply s2
                     s1 = rightSupply s2
    in s0 `seq` (s1 `seq` C_TermCons (Curry_Prelude.nd_C_apply x1 x4 s0 cd cs)
    (Curry_Prelude.nd_C_map (wrapNX id (nd_C_mapTerm x1)) x5 s1 cd cs)))
  Choice_C_Term d i l r -> narrow d i (nd_C_mapTerm x1 l s cd cs) (nd_C_mapTerm
    x1 r s cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> nd_C_mapTerm x1 z s cd cs) xs
  Guard_C_Term d c e -> guardCons d c (nd_C_mapTerm x1 e s cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term.mapTerm" [show x1
    , show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term.mapTerm" (showCons x2))

d_C_eqConsPattern :: forall t0 . (Curry_Prelude.Curry t0) => C_Term t0 -> C_Term
  t0 -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_eqConsPattern x1 x2 cd cs = case x1 of
  C_TermVar x3 -> Curry_Prelude.C_False
  C_TermCons x4 x5 -> d_C__case_2 x5 x4 x2 cd cs
  Choice_C_Term d i l r -> narrow d i (d_C_eqConsPattern l x2 cd cs)
    (d_C_eqConsPattern r x2 cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C_eqConsPattern z x2 cd cs)
    xs
  Guard_C_Term d c e -> guardCons d c (d_C_eqConsPattern e x2 cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term.eqConsPattern"
    [show x1, show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term.eqConsPattern" (showCons x1))

d_C_parensIf :: Curry_Prelude.C_Bool -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
d_C_parensIf x1 x2 cd cs = case x1 of
  Curry_Prelude.C_True -> Curry_Prelude.d_OP_plus_plus
    (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus x2
    (Curry_Prelude.toCurryString ")") cd cs) cd cs
  Curry_Prelude.C_False -> x2
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (d_C_parensIf l x2 cd cs)
    (d_C_parensIf r x2 cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> d_C_parensIf z x2
    cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (d_C_parensIf e x2 cd $!
    addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "Rewriting.Term.parensIf" [show x1, show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term.parensIf" (showCons x1))

d_C_unique :: forall t0 . (Curry_Prelude.Curry t0) => Curry_Prelude.OP_List t0
  -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C_unique x1 cd cs = case x1 of
  Curry_Prelude.OP_List -> Curry_Prelude.C_True
  Curry_Prelude.OP_Cons x2 x3 -> d_C__case_1 x3 x2 (Curry_Prelude.d_C_apply
    (Curry_Prelude.d_C_notElem x2 cd cs) x3 cd cs) cd cs
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (d_C_unique l cd cs)
    (d_C_unique r cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> d_C_unique z cd
    cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (d_C_unique e cd $! addCs c
    cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term.unique" [show x1] info)
  _ -> failCons cd (consFail "Rewriting.Term.unique" (showCons x1))

d_C__case_1 :: forall t0 . (Curry_Prelude.Curry t0) => Curry_Prelude.OP_List t0
  -> t0 -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_1 x3 x2 x4 cd cs = case x4 of
  Curry_Prelude.C_True -> d_C_unique x3 cd cs
  Curry_Prelude.C_False -> d_C__case_0 (Curry_Prelude.d_C_otherwise cd cs) cd cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (d_C__case_1 x3 x2 l cd cs)
    (d_C__case_1 x3 x2 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> d_C__case_1 x3 x2
    z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (d_C__case_1 x3 x2 e cd $!
    addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "Rewriting.Term._case_1" [show x3, show x2, show x4] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_1" (showCons x4))

d_C__case_0 :: Curry_Prelude.C_Bool -> Cover -> ConstStore
  -> Curry_Prelude.C_Bool
d_C__case_0 x1 cd cs = case x1 of
  Curry_Prelude.C_True -> Curry_Prelude.C_False
  Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (d_C__case_0 l cd cs)
    (d_C__case_0 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> d_C__case_0 z cd
    cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (d_C__case_0 e cd $! addCs c
    cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "Rewriting.Term._case_0" [show x1] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_0" (showCons x1))

d_C__case_2 :: forall t0 . (Curry_Prelude.Curry t0) => Curry_Prelude.OP_List
  (C_Term t0) -> t0 -> C_Term t0 -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_2 x5 x4 x2 cd cs = case x2 of
  C_TermVar x6 -> Curry_Prelude.C_False
  C_TermCons x7 x8 -> Curry_Prelude.d_OP_amp_amp (Curry_Prelude.d_OP_eq_eq x4 x7
    cd cs) (Curry_Prelude.d_OP_eq_eq (Curry_Prelude.d_C_length x5 cd cs)
    (Curry_Prelude.d_C_length x8 cd cs) cd cs) cd cs
  Choice_C_Term d i l r -> narrow d i (d_C__case_2 x5 x4 l cd cs) (d_C__case_2
    x5 x4 r cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C__case_2 x5 x4 z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_C__case_2 x5 x4 e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term._case_2" [show x5
    , show x4, show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_2" (showCons x2))

d_C__case_3 :: Curry_Prelude.OP_List Curry_Prelude.C_Int -> Cover -> ConstStore
  -> Curry_Prelude.C_Maybe Curry_Prelude.C_Int
d_C__case_3 x2 cd cs = case x2 of
  Curry_Prelude.OP_List -> Curry_Prelude.C_Nothing
  Curry_Prelude.OP_Cons x3 x4 -> Curry_Prelude.C_Just (Curry_List.d_C_minimum x2
    cd cs)
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (d_C__case_3 l cd cs)
    (d_C__case_3 r cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> d_C__case_3 z cd
    cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (d_C__case_3 e cd $! addCs
    c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term._case_3" [show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_3" (showCons x2))

d_C__case_4 :: Curry_Prelude.OP_List Curry_Prelude.C_Int -> Cover -> ConstStore
  -> Curry_Prelude.C_Maybe Curry_Prelude.C_Int
d_C__case_4 x2 cd cs = case x2 of
  Curry_Prelude.OP_List -> Curry_Prelude.C_Nothing
  Curry_Prelude.OP_Cons x3 x4 -> Curry_Prelude.C_Just (Curry_List.d_C_maximum x2
    cd cs)
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (d_C__case_4 l cd cs)
    (d_C__case_4 r cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> d_C__case_4 z cd
    cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (d_C__case_4 e cd $! addCs
    c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term._case_4" [show x2] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_4" (showCons x2))

d_C__case_6 :: forall t0 . (Curry_Prelude.Curry t0) => t0
  -> Curry_Prelude.OP_List (C_Term t0) -> Cover -> ConstStore
  -> Curry_Prelude.C_Bool
d_C__case_6 x3 x4 cd cs = case x4 of
  Curry_Prelude.OP_List -> Curry_Prelude.C_True
  Curry_Prelude.OP_Cons x5 x6 -> d_C__case_5 x6 x3 x5 cd cs
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (d_C__case_6 x3 l cd cs)
    (d_C__case_6 x3 r cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> d_C__case_6 x3 z
    cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (d_C__case_6 x3 e cd $!
    addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term._case_6" [show x3, show x4] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_6" (showCons x4))

d_C__case_5 :: forall t0 . (Curry_Prelude.Curry t0) => Curry_Prelude.OP_List
  (C_Term t0) -> t0 -> C_Term t0 -> Cover -> ConstStore -> Curry_Prelude.C_Bool
d_C__case_5 x6 x3 x5 cd cs = case x5 of
  C_TermVar x7 -> Curry_Prelude.d_C_apply (Curry_Prelude.d_C_all (d_C_isVarTerm
    cd cs) cd cs) x6 cd cs
  C_TermCons x8 x9 -> Curry_Prelude.d_OP_amp_amp (d_C_isNormal x5 cd cs)
    (d_C_isNormal (C_TermCons x3 x6) cd cs) cd cs
  Choice_C_Term d i l r -> narrow d i (d_C__case_5 x6 x3 l cd cs) (d_C__case_5
    x6 x3 r cd cs)
  Choices_C_Term d i xs -> narrows cs d i (\z -> d_C__case_5 x6 x3 z cd cs) xs
  Guard_C_Term d c e -> guardCons d c (d_C__case_5 x6 x3 e cd $! addCs c cs)
  Fail_C_Term d info -> failCons d (traceFail "Rewriting.Term._case_5" [show x6
    , show x3, show x5] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_5" (showCons x5))

d_C__case_9 :: forall t0 . (Curry_Prelude.Curry t0) => (t0 -> Cover
  -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char) -> t0
  -> Curry_Prelude.C_Bool -> Curry_Prelude.OP_List (C_Term t0) -> Cover
  -> ConstStore -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_9 x1 x5 x2 x6 cd cs = case x6 of
  Curry_Prelude.OP_List -> Curry_Prelude.d_C_apply x1 x5 cd cs
  Curry_Prelude.OP_Cons x7 x8 -> d_C__case_8 x6 x1 x5 x7 x2 x8 cd cs
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (d_C__case_9 x1 x5 x2 l cd
    cs) (d_C__case_9 x1 x5 x2 r cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> d_C__case_9 x1
    x5 x2 z cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (d_C__case_9 x1 x5 x2 e
    cd $! addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term._case_9" [show x1, show x5, show x2, show x6] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_9" (showCons x6))

nd_C__case_9 :: forall t0 . (Curry_Prelude.Curry t0) => Func t0
  (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> t0 -> Curry_Prelude.C_Bool
  -> Curry_Prelude.OP_List (C_Term t0) -> IDSupply -> Cover -> ConstStore
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char
nd_C__case_9 x1 x5 x2 x6 s cd cs = case x6 of
  Curry_Prelude.OP_List -> let s0 = s
    in s0 `seq` Curry_Prelude.nd_C_apply x1 x5 s0 cd cs
  Curry_Prelude.OP_Cons x7 x8 -> let s0 = s
    in s0 `seq` nd_C__case_8 x6 x1 x5 x7 x2 x8 s0 cd cs
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (nd_C__case_9 x1 x5 x2 l s
    cd cs) (nd_C__case_9 x1 x5 x2 r s cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> nd_C__case_9 x1
    x5 x2 z s cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (nd_C__case_9 x1 x5 x2 e s
    cd $! addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term._case_9" [show x1, show x5, show x2, show x6] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_9" (showCons x6))

d_C__case_8 :: forall t0 . (Curry_Prelude.Curry t0) => Curry_Prelude.OP_List
  (C_Term t0) -> (t0 -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char) -> t0 -> C_Term t0 -> Curry_Prelude.C_Bool
  -> Curry_Prelude.OP_List (C_Term t0) -> Cover -> ConstStore
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_8 x6 x1 x5 x7 x2 x8 cd cs = case x8 of
  Curry_Prelude.OP_Cons x9 x10 -> d_C__case_7 x6 x1 x5 x9 x7 x2 x10 cd cs
  Curry_Prelude.OP_List -> Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_apply
    x1 x5 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString "(")
    (Curry_Prelude.d_OP_plus_plus (Curry_List.d_C_intercalate
    (Curry_Prelude.toCurryString ",") (Curry_Prelude.d_C_map
    (d_OP_showTerm_dot_showTerm_tick_dot_9 x1 Curry_Prelude.C_False) x6 cd cs)
    cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (d_C__case_8 x6 x1 x5 x7 x2
    l cd cs) (d_C__case_8 x6 x1 x5 x7 x2 r cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> d_C__case_8 x6
    x1 x5 x7 x2 z cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (d_C__case_8 x6 x1 x5 x7 x2
    e cd $! addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term._case_8" [show x6, show x1, show x5, show x7, show x2, show
    x8] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_8" (showCons x8))

nd_C__case_8 :: forall t0 . (Curry_Prelude.Curry t0) => Curry_Prelude.OP_List
  (C_Term t0) -> Func t0 (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> t0
  -> C_Term t0 -> Curry_Prelude.C_Bool -> Curry_Prelude.OP_List (C_Term t0)
  -> IDSupply -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
nd_C__case_8 x6 x1 x5 x7 x2 x8 s cd cs = case x8 of
  Curry_Prelude.OP_Cons x9 x10 -> let s0 = s
    in s0 `seq` nd_C__case_7 x6 x1 x5 x9 x7 x2 x10 s0 cd cs
  Curry_Prelude.OP_List -> let s2 = s
    in s2 `seq` (let s0 = leftSupply s2
                     s1 = rightSupply s2
    in s0 `seq` (s1 `seq` Curry_Prelude.d_OP_plus_plus (Curry_Prelude.nd_C_apply
    x1 x5 s0 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString
    "(") (Curry_Prelude.d_OP_plus_plus (Curry_List.d_C_intercalate
    (Curry_Prelude.toCurryString ",") (Curry_Prelude.nd_C_map (wrapNX id
    (nd_OP_showTerm_dot_showTerm_tick_dot_9 x1 Curry_Prelude.C_False)) x6 s1 cd
    cs) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs))
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (nd_C__case_8 x6 x1 x5 x7
    x2 l s cd cs) (nd_C__case_8 x6 x1 x5 x7 x2 r s cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> nd_C__case_8 x6
    x1 x5 x7 x2 z s cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (nd_C__case_8 x6 x1 x5 x7
    x2 e s cd $! addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term._case_8" [show x6, show x1, show x5, show x7, show x2, show
    x8] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_8" (showCons x8))

d_C__case_7 :: forall t0 . (Curry_Prelude.Curry t0) => Curry_Prelude.OP_List
  (C_Term t0) -> (t0 -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char) -> t0 -> C_Term t0 -> C_Term t0 -> Curry_Prelude.C_Bool
  -> Curry_Prelude.OP_List (C_Term t0) -> Cover -> ConstStore
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_7 x6 x1 x5 x9 x7 x2 x10 cd cs = case x10 of
  Curry_Prelude.OP_List -> d_C_parensIf x2 (Curry_Prelude.d_OP_plus_plus
    (d_OP_showTerm_dot_showTerm_tick_dot_9 x1 Curry_Prelude.C_True x7 cd cs)
    (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ")
    (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.d_C_apply x1 x5 cd cs)
    (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ")
    (d_OP_showTerm_dot_showTerm_tick_dot_9 x1 Curry_Prelude.C_True x9 cd cs) cd
    cs) cd cs) cd cs) cd cs) cd cs
  Curry_Prelude.OP_Cons x11 x12 -> Curry_Prelude.d_OP_plus_plus
    (Curry_Prelude.d_C_apply x1 x5 cd cs) (Curry_Prelude.d_OP_plus_plus
    (Curry_Prelude.toCurryString "(") (Curry_Prelude.d_OP_plus_plus
    (Curry_List.d_C_intercalate (Curry_Prelude.toCurryString ",")
    (Curry_Prelude.d_C_map (d_OP_showTerm_dot_showTerm_tick_dot_9 x1
    Curry_Prelude.C_False) x6 cd cs) cd cs) (Curry_Prelude.toCurryString ")") cd
    cs) cd cs) cd cs
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (d_C__case_7 x6 x1 x5 x9 x7
    x2 l cd cs) (d_C__case_7 x6 x1 x5 x9 x7 x2 r cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> d_C__case_7 x6
    x1 x5 x9 x7 x2 z cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (d_C__case_7 x6 x1 x5 x9 x7
    x2 e cd $! addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term._case_7" [show x6, show x1, show x5, show x9, show x7, show
    x2, show x10] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_7" (showCons x10))

nd_C__case_7 :: forall t0 . (Curry_Prelude.Curry t0) => Curry_Prelude.OP_List
  (C_Term t0) -> Func t0 (Curry_Prelude.OP_List Curry_Prelude.C_Char) -> t0
  -> C_Term t0 -> C_Term t0 -> Curry_Prelude.C_Bool -> Curry_Prelude.OP_List
  (C_Term t0) -> IDSupply -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
nd_C__case_7 x6 x1 x5 x9 x7 x2 x10 s cd cs = case x10 of
  Curry_Prelude.OP_List -> let s4 = s
    in s4 `seq` d_C_parensIf x2 (let s0 = leftSupply s4
                                     s3 = rightSupply s4
    in s0 `seq` (s3 `seq` Curry_Prelude.d_OP_plus_plus
    (nd_OP_showTerm_dot_showTerm_tick_dot_9 x1 Curry_Prelude.C_True x7 s0 cd cs)
    (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString " ")
    (let s1 = leftSupply s3
         s2 = rightSupply s3
    in s1 `seq` (s2 `seq` Curry_Prelude.d_OP_plus_plus (Curry_Prelude.nd_C_apply
    x1 x5 s1 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString
    " ") (nd_OP_showTerm_dot_showTerm_tick_dot_9 x1 Curry_Prelude.C_True x9 s2
    cd cs) cd cs) cd cs)) cd cs) cd cs)) cd cs
  Curry_Prelude.OP_Cons x11 x12 -> let s2 = s
    in s2 `seq` (let s0 = leftSupply s2
                     s1 = rightSupply s2
    in s0 `seq` (s1 `seq` Curry_Prelude.d_OP_plus_plus (Curry_Prelude.nd_C_apply
    x1 x5 s0 cd cs) (Curry_Prelude.d_OP_plus_plus (Curry_Prelude.toCurryString
    "(") (Curry_Prelude.d_OP_plus_plus (Curry_List.d_C_intercalate
    (Curry_Prelude.toCurryString ",") (Curry_Prelude.nd_C_map (wrapNX id
    (nd_OP_showTerm_dot_showTerm_tick_dot_9 x1 Curry_Prelude.C_False)) x6 s1 cd
    cs) cd cs) (Curry_Prelude.toCurryString ")") cd cs) cd cs) cd cs))
  Curry_Prelude.Choice_OP_List d i l r -> narrow d i (nd_C__case_7 x6 x1 x5 x9
    x7 x2 l s cd cs) (nd_C__case_7 x6 x1 x5 x9 x7 x2 r s cd cs)
  Curry_Prelude.Choices_OP_List d i xs -> narrows cs d i (\z -> nd_C__case_7 x6
    x1 x5 x9 x7 x2 z s cd cs) xs
  Curry_Prelude.Guard_OP_List d c e -> guardCons d c (nd_C__case_7 x6 x1 x5 x9
    x7 x2 e s cd $! addCs c cs)
  Curry_Prelude.Fail_OP_List d info -> failCons d (traceFail
    "Rewriting.Term._case_7" [show x6, show x1, show x5, show x9, show x7, show
    x2, show x10] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_7" (showCons x10))

d_C__case_12 :: Curry_Prelude.C_Int -> Curry_Prelude.C_Int
  -> Curry_Prelude.C_Char -> Curry_Prelude.C_Bool -> Cover -> ConstStore
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_12 x1 x3 x5 x6 cd cs = case x6 of
  Curry_Prelude.C_True -> d_C__case_11 x3 x5 (Curry_Prelude.d_OP_eq_eq x3
    (Curry_Prelude.C_Int 0#) cd cs) cd cs
  Curry_Prelude.C_False -> d_C__case_10 (Curry_Prelude.d_C_otherwise cd cs) cd
    cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (d_C__case_12 x1 x3 x5 l cd
    cs) (d_C__case_12 x1 x3 x5 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> d_C__case_12 x1
    x3 x5 z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (d_C__case_12 x1 x3 x5 e
    cd $! addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "Rewriting.Term._case_12" [show x1, show x3, show x5, show x6] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_12" (showCons x6))

d_C__case_10 :: Curry_Prelude.C_Bool -> Cover -> ConstStore
  -> Curry_Prelude.OP_List Curry_Prelude.C_Char
d_C__case_10 x1 cd cs = case x1 of
  Curry_Prelude.C_True -> Curry_Prelude.OP_List
  Curry_Prelude.C_False -> Curry_Prelude.d_C_failed cd cs
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (d_C__case_10 l cd cs)
    (d_C__case_10 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> d_C__case_10 z cd
    cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (d_C__case_10 e cd $! addCs
    c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "Rewriting.Term._case_10" [show x1] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_10" (showCons x1))

d_C__case_11 :: Curry_Prelude.C_Int -> Curry_Prelude.C_Char
  -> Curry_Prelude.C_Bool -> Cover -> ConstStore -> Curry_Prelude.OP_List
  Curry_Prelude.C_Char
d_C__case_11 x3 x5 x6 cd cs = case x6 of
  Curry_Prelude.C_True -> Curry_Prelude.OP_Cons x5 Curry_Prelude.OP_List
  Curry_Prelude.C_False -> Curry_Prelude.OP_Cons x5 (Curry_Prelude.d_C_show x3
    cd cs)
  Curry_Prelude.Choice_C_Bool d i l r -> narrow d i (d_C__case_11 x3 x5 l cd cs)
    (d_C__case_11 x3 x5 r cd cs)
  Curry_Prelude.Choices_C_Bool d i xs -> narrows cs d i (\z -> d_C__case_11 x3
    x5 z cd cs) xs
  Curry_Prelude.Guard_C_Bool d c e -> guardCons d c (d_C__case_11 x3 x5 e cd $!
    addCs c cs)
  Curry_Prelude.Fail_C_Bool d info -> failCons d (traceFail
    "Rewriting.Term._case_11" [show x3, show x5, show x6] info)
  _ -> failCons cd (consFail "Rewriting.Term._case_11" (showCons x6))

